%%%%%%%%%%%%%%%%%%  REQUIRES WaveLab Toolbox   %%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Signals pertaining to Class \Theta according to the SNR (see readme) %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
SigName={'HeaviSine' 'Bumps' 'Blocks' 'Doppler' 'Ramp' 'Cusp' 'Sing' 'HiSine' 'LoSine' 'LinChirp' 'Piece-Polynomial' 'QuadChirp' 'MishMash' 'WernerSorrows' 'Leopold' 'Piece-Regular' 'Riemann' 'HypChirps' 'sineoneoverx' 'Chirps'}; 
Lsn=length(SigName);
N = 2^11;
zetaN=N^(-1/4)*(1+sqrt(1-1/N^2))^(-1/2)*exp(-(log(1+sqrt(1-1/N^2)))^2 / (4*log(N)))*ones(Lsn,1); 
j0=10;   % N=2^(j+1) % Decomposition level
qmf=MakeONFilter('Symmlet',8);
J=log2(N)-j0;  % resolution level in wavelab sense
CLASS_THETA=zeros(Lsn, 4);
%%%%%%%%%%%%%%%%%%%%
for m=1:Lsn
    SName=char(SigName(m));
    Sig=MakeSignal(SName,N);
    rand('seed',0);
    for k=1:4   % RSB
        rho=2*k-1;
        [xSig,ySig] = NoiseMaker(Sig,rho);
        wc = FWT_PO(xSig,J,qmf);
        CLASS_THETA(m,k)=(length(find(abs(wc)>=1))+sum(wc(find(abs(wc)<1)).^2))/N;   % r0(\theta)
        
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot(CLASS_THETA(:,1),'o')
hold on
plot(CLASS_THETA(:,2),'rs')
hold on
plot(CLASS_THETA(:,3),'md')
hold on
plot(CLASS_THETA(:,4),'gh')
hold on
plot(zetaN,'k:+')
hold off
xlabel('Signal index')
ylabel('r_0(\theta)')
legend('SNR=1','SNR=3','SNR=5','SNR=7','\zeta(N)')
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright: am.atto@telecom-bretagne.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%