%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Denoising of 2D signals by using Discrete Wavelet Transform (DWT2) and 
%  soft thresholding with universal, minimax, and detection thresholds:
% the images that are processed are
% {'testimages/house.png' 'testimages/peppers256.png' 
% 'testimages/barbara.png' 'testimages/lena.png' 
% 'testimages/fingerprint.png' 'testimages/boat.png'}
% are this demo returns the PSNRs and estimation risks for these images
% when noise is additive, white and gaussian with standard deviation sbn =
% 25 (default value).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%        Risks and PSNR comparison       %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
close all
clear all
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load original images
ImgName={'testimages/house.png' 'testimages/peppers256.png' 'testimages/barbara.png' 'testimages/lena.png' 'testimages/fingerprint.png' 'testimages/boat.png'}; 
Lsn=length(ImgName);
j0=4;   % N=2^(j+1) % Decomposition level
sbn=25;
ITER=1;   % Number of Iteration (Monte Carlo)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PSNR_0=zeros(1,Lsn);  % initial PSNR
PSNR_U=zeros(1,Lsn);  % Universal Soft thresholding estimate PNSR 
PSNR_M=zeros(1,Lsn);  % Minimax Soft thresholding estimate PNSR 
PSNR_P=zeros(1,Lsn);  % Detection Soft thresholding estimate PNSR 
%
rsqU=zeros(1,Lsn);    % Risks, Universal
rsqM=zeros(1,Lsn);    % Risks, Minimax
rsqP=zeros(1,Lsn);    % Risks, Detection
%
Wname='sym8';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
for m=1:Lsn   % for every image given in ImgName
    IName=char(ImgName(m));
    im0 = imread(IName);
    mini=min(min(im0));
    if mini<0
        im0=im0+abs(mini);
    end
    img0 = double(im0);  % convert to double
    [Ny,Nx] = size(im0);
    Lsim=Nx*Ny;
    r0=reshape(img0,1,Lsim);
    for IndexIter=1:ITER
        seed = 0;               % random seed
        randn('state', seed);
        noise = randn(size(img0));
        noise = noise/sqrt(mean2(noise.^2));
        imb = img0 + sbn*noise;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        SNR_0 = snr(img0 - mean2(img0), imb - img0);
        PSNR_0(m) = PSNR_0(m) + SNR_0 + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%   DECOMPOSITION  %%%%%%%%%%%%%%%%%%%
        %tic; 
        [imaUdeb, imaMdeb, imaPdeb]=CompSoftThDWT2(imb, sbn, j0, Wname); 
        %toc
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        SNR_U = snr(img0 - mean2(img0), imaUdeb - img0);
        PSNR_U(m) = PSNR_U(m) + SNR_U + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        SNR_M = snr(img0 - mean2(img0), imaMdeb - img0);
        PSNR_M(m) = PSNR_M(m) + SNR_M + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        SNR_P = snr(img0 - mean2(img0), imaPdeb - img0);
        PSNR_P(m) = PSNR_P(m) + SNR_P + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        rU=reshape(imaUdeb,1,Lsim);
        rM=reshape(imaMdeb,1,Lsim);
        rP=reshape(imaPdeb,1,Lsim);
        rsqU(m)=rsqU(m)+(1/Lsim)*((rU-r0)*(rU-r0)');
        rsqM(m)=rsqM(m)+(1/Lsim)*((rM-r0)*(rM-r0)');
        rsqP(m)=rsqP(m)+(1/Lsim)*((rP-r0)*(rP-r0)');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    end    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PSNR_U=PSNR_U./ITER
PSNR_M=PSNR_M./ITER
PSNR_P=PSNR_P./ITER
rsqU=rsqU./ITER
rsqM=rsqM./ITER
rsqP=rsqP./ITER
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright: am.atto@telecom-bretagne.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%