%%%%%%%%%%%%%%%%%%  REQUIRES WaveLab Toolbox   %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Denoising of 1D signals by using Stationary Wavelet Transform and 
%  soft thresholding with universal, minimax, and detection thresholds.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SigName={'HeaviSine' 'Bumps' 'Blocks' 'Doppler' 'Ramp' 'Cusp' 'Sing' 'HiSine' 'LoSine' 'LinChirp' 'Piece-Polynomial' 'QuadChirp' 'MishMash' 'WernerSorrows' 'Leopold' 'Piece-Regular' 'Riemann' 'HypChirps' 'sineoneoverx' 'Chirps'}; 
Lsn=length(SigName);
m=4; % index of the analysed signal (in SigName): 1<=m<=Lsn (m=4->Blocks) 
N = 2^11;
WName='sym8'; % Wavelet used.
rho = 7;    % SNR
j0=6;       % Maximum le vel concerned by thresholding, j <= 10.
%%%%%%% Thresholds %%%%%%%%%%%%%%%
lambdaU=sqrt(2*log(N)); % Unversal
lambdaM = 0.3936 + 0.1829*(log(N)/log(2)); % Standard approximation of the Minimax threshold
lambdaP=0.5*lambdaU+log(1+sqrt(1-1/(N)^2))/lambdaU; % Detection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SName=char(SigName(m));
Sig=MakeSignal(SName,N);
rand('seed',0);
[xSig,ySig] = NoiseMaker(Sig,rho);
%%%%%%%%%%%  SWT  %%%%%%%%%%%%%%%%%%%%%%%%%%
swc=swt(ySig,j0,WName);
swcP=swc;
swcM=swc;
swcU=swc;
for ind=1:j0
    swcP(ind,:)=SoftThresh(swc(ind,:),lambdaP);
    swcM(ind,:)=SoftThresh(swc(ind,:),lambdaM);
    swcU(ind,:)=SoftThresh(swc(ind,:),lambdaU);
end
denSigP=iswt(swcP,WName);
denSigM=iswt(swcM,WName);
denSigU=iswt(swcU,WName);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% FIGURES
figure
plot(xSig,'r:')
hold
plot(denSigP,'b');
hold off
legend('Original','Detection')
%
figure
plot(xSig,'r:')
hold
plot(denSigM,'b')
hold off
legend('Original','Minimax')
%
figure
plot(xSig,'r:')
hold
plot(denSigU,'b')
hold off
legend('Original','Universal')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%  ZOOM 
% 50 first coeffs
T=1:50;
figure
plot(xSig(T),'r:')
hold on
plot(denSigP(T),'b');
hold on
plot(denSigM(T),'g')
hold on
plot(denSigU(T),'m')
hold off
legend('Original','Detection','Minimax','Universal')
% 50 last coeffs
T=N-50:N;
figure
plot(T,xSig(T),'r:')
hold on
plot(T,denSigP(T),'b');
hold on
plot(T,denSigM(T),'g')
hold on
plot(T,denSigU(T),'m')
hold off
legend('Original','Detection','Minimax','Universal')
%%%%%%%%   END ZOOM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright: am.atto@telecom-bretagne.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%