%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Denoising of 2D signals by using Stationary Wavelet Transform and 
%  soft thresholding with universal, minimax, and detection thresholds.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
clc
close all
clear all
PSNR_0=0;  % Initial PSNR
PSNR_U=0;  % Universal Soft thresholding estimate PNSR 
PSNR_M=0;  % Minimax Soft thresholding estimate PNSR 
PSNR_P=0;  % Detection Soft thresholding estimate PNSR 
PSNR_A=0;  % Level Dependent Detection Soft thresholding estimate PNSR
Wname='db1';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%   Load image
%im0 = imread('testimages/barbara.png');
%im0 = imread('testimages/boat.png');
%im0 = imread('testimages/fingerprint.png');
%im0 = imread('testimages/flinstones.png');
%im0 = imread('testimages/house.png');
im0 = imread('testimages/lena.png');
%im0 = imread('testimages/peppers256.png');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mini=min(min(im0));
if mini<0
    im0=im0+abs(mini);
end
img0 = double(im0);  % convert to double
sim=size(img0);
Lsim=sim(1)*sim(2);
%%%%%%%%%%%%%%%%%%
sbn=25;   % Noise standard deviation
j0=4;   % N=2^(j+1) % Decomposition level
ITER=1;      % Number of Iteration (Monte Carlo)
for IndexIter=1:ITER
    noise = randn(sim);
    imb = img0 + sbn*noise;
    %%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    SNR_0 = snr(img0 - mean2(img0), imb - img0);
    PSNR_0 = PSNR_0 + SNR_0 + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%  Comparaison U-M-P-A avec le SOFT %%%%%%%
    tic; 
    [imaUdeb, imaMdeb, imaPdeb imaAdeb]=CompSoftThSWT2(imb, sbn, j0, Wname); 
    toc
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    SNR_U = snr(img0 - mean2(img0), imaUdeb - img0);
    PSNR_U = PSNR_U+SNR_U + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
    %ISNR_U = SNR_U - SNR_0;   % increment of the SNR
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    SNR_M = snr(img0 - mean2(img0), imaMdeb - img0);
    PSNR_M = PSNR_M+SNR_M + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
    %ISNR_M = SNR_M - SNR_0;    % increment of the SNR
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    SNR_P = snr(img0 - mean2(img0), imaPdeb - img0);
    PSNR_P = PSNR_P + SNR_P + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
    %ISNR_P = SNR_P - SNR_0;     % increment of the SNR
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    SNR_A = snr(img0 - mean2(img0), imaAdeb - img0);
    PSNR_A = PSNR_A + SNR_A + 10*log10(255^2/mean2((img0-mean2(img0)).^2));
    %ISNR_A = SNR_A - SNR_0;     % increment of the SNR
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
PSNR_0=PSNR_0/ITER   % Initial PSNR
PSNR_U=PSNR_U/ITER   % Universal Soft thresholding estimate PNSR
PSNR_M=PSNR_M/ITER   % Minimax Soft thresholding estimate PNSR 
PSNR_P=PSNR_P/ITER   % Detection Soft thresholding estimate PNSR 
PSNR_A=PSNR_A/ITER   % Level Dependent Detection Soft thresholding estimate PNSR
%%%%%%%%%%%%%%%%%%%%%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure
% colormap(gray)
% imagesc(img0)
% title('Original Image')
% %%%%%%%%%%%%%%%%%
figure
colormap(gray)
imagesc(imb)
title('Noisy Image')
%%%%%%%%%%%%%%%%%%
figure
colormap(gray)
imagesc(imaUdeb)
title('Soft - Universal')
%%%%%%%%%%%%%%%%%%
figure
colormap(gray)
imagesc(imaMdeb)
title('Soft - Minimax')
%%%%%%%%%%%%%%%%%%
figure
colormap(gray)
imagesc(imaPdeb)
title('Soft - Detection')
%%%%%%%%%%%%%%%%%%
figure
colormap(gray)
imagesc(imaAdeb)
title('Soft - LD Detection')  % Level Dependent (LD) Detection
%%%%%%%%%%%%%%%%%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright: am.atto@telecom-bretagne.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%